var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { CharStreams, CommonTokenStream } from 'antlr4ng';
import * as c3 from 'antlr4-c3';
import { findCursorTokenIndex, getCursorIndex, } from '../../lib/cursor.js';
import { getCurrentStatement, modifyInvalidQuery, spaceSymbols } from '../../lib/query.js';
import { TableSymbol } from '../../lib/symbolTable.js';
import { SqlErrorListener } from '../../lib/sqlErrorListener.js';
import { MySqlLexer } from './generated/MySqlLexer.js';
import { MySqlParser } from './generated/MySqlParser.js';
import { MySqlParserVisitor } from './generated/MySqlParserVisitor.js';
import { preferredRules } from './lib/preferredRules.js';
import { ignoredTokens } from './lib/ignoredTokens.js';
var TableSuggestion;
(function (TableSuggestion) {
    TableSuggestion["ALL"] = "ALL";
    TableSuggestion["TABLES"] = "TABLES";
    TableSuggestion["VIEWS"] = "VIEWS";
})(TableSuggestion || (TableSuggestion = {}));
const quotesRegex = /^'(.*)'$/;
class SymbolTableVisitor extends MySqlParserVisitor {
    constructor() {
        super();
        this.visitTableName = (context) => {
            try {
                this.symbolTable.addNewSymbolOfType(TableSymbol, this.scope, context.getText());
            }
            catch (error) {
                if (!(error instanceof c3.DuplicateSymbolError)) {
                    throw error;
                }
            }
            return this.visitChildren(context);
        };
        this.visitAtomTableItem = (context) => {
            var _a;
            try {
                const rawAlias = (_a = context.uid()) === null || _a === void 0 ? void 0 : _a.getText();
                // For some reason LEFT | RIGHT keyword gets confused with alias
                const isAliasPartOfJoinStatement = (rawAlias === null || rawAlias === void 0 ? void 0 : rawAlias.toLowerCase()) === 'left' || (rawAlias === null || rawAlias === void 0 ? void 0 : rawAlias.toLowerCase()) === 'right';
                this.symbolTable.addNewSymbolOfType(TableSymbol, this.scope, context.tableName().getText(), isAliasPartOfJoinStatement ? undefined : rawAlias);
            }
            catch (error) {
                if (!(error instanceof c3.DuplicateSymbolError)) {
                    throw error;
                }
            }
            return this.visitChildren(context);
        };
        this.symbolTable = new c3.SymbolTable('', {});
        this.scope = this.symbolTable.addNewSymbolOfType(c3.ScopedSymbol, undefined);
    }
}
const explainRegex = new RegExp(`^(${spaceSymbols})?explain${spaceSymbols}$`);
const multipleKeywordsRegex = new RegExp(`^(${spaceSymbols})?\\S+${spaceSymbols}`);
function shouldSuggestTemplates(statement, cursorIndex) {
    const currentStatementBeforeCursor = statement.slice(0, cursorIndex).toLowerCase();
    return Boolean(cursorIndex === 0 ||
        // First keyword in statement
        !currentStatementBeforeCursor.match(multipleKeywordsRegex) ||
        // Explain statement
        currentStatementBeforeCursor.match(explainRegex));
}
function generateSuggestionsFromRules(rules, cursorTokenIndex, previousToken) {
    var _a, _b;
    let suggestTables;
    let suggestAggregateFunctions = false;
    let suggestFunctions = false;
    let suggestColumns = false;
    for (const [ruleId, ruleData] of rules) {
        switch (ruleId) {
            case MySqlParser.RULE_tableName: {
                if (cursorTokenIndex === ruleData.startTokenIndex &&
                    !ruleData.ruleList.includes(MySqlParser.RULE_createTable)) {
                    suggestTables = TableSuggestion.ALL;
                }
                break;
            }
            case MySqlParser.RULE_alterTable:
            case MySqlParser.RULE_dropTable: {
                const isPreviousTokenTable = ((_a = previousToken === null || previousToken === void 0 ? void 0 : previousToken.text) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === 'table';
                if (isPreviousTokenTable) {
                    suggestTables = TableSuggestion.TABLES;
                }
                break;
            }
            case MySqlParser.RULE_alterView:
            case MySqlParser.RULE_dropView: {
                const isPreviousTokenView = ((_b = previousToken === null || previousToken === void 0 ? void 0 : previousToken.text) === null || _b === void 0 ? void 0 : _b.toLowerCase()) === 'view';
                if (isPreviousTokenView) {
                    suggestTables = TableSuggestion.VIEWS;
                }
                break;
            }
            case MySqlParser.RULE_aggregateWindowedFunction: {
                suggestAggregateFunctions = true;
                break;
            }
            case MySqlParser.RULE_scalarFunctionName: {
                suggestFunctions = true;
                break;
            }
            case MySqlParser.RULE_fullColumnName:
            case MySqlParser.RULE_indexColumnName: {
                if (cursorTokenIndex === ruleData.startTokenIndex) {
                    suggestColumns = true;
                }
                break;
            }
        }
    }
    return { suggestTables, suggestAggregateFunctions, suggestFunctions, suggestColumns };
}
export function parseMySqlQueryWithoutCursor(query) {
    const inputStream = CharStreams.fromString(query);
    const lexer = new MySqlLexer(inputStream);
    const tokenStream = new CommonTokenStream(lexer);
    const parser = new MySqlParser(tokenStream);
    const errorListener = new SqlErrorListener(MySqlLexer.SPACE);
    parser.removeErrorListeners();
    parser.addErrorListener(errorListener);
    parser.root();
    return { errors: errorListener.errors };
}
export function parseMySqlQuery(query, cursor) {
    const inputStream = CharStreams.fromString(query);
    const lexer = new MySqlLexer(inputStream);
    const tokenStream = new CommonTokenStream(lexer);
    const parser = new MySqlParser(tokenStream);
    const errorListener = new SqlErrorListener(MySqlLexer.SPACE);
    parser.removeErrorListeners();
    parser.addErrorListener(errorListener);
    parser.root();
    const core = new c3.CodeCompletionCore(parser);
    core.ignoredTokens = ignoredTokens;
    core.preferredRules = preferredRules;
    const cursorTokenIndex = findCursorTokenIndex(tokenStream, cursor, MySqlLexer.SPACE);
    const suggestKeywords = [];
    let result = {
        errors: errorListener.errors,
    };
    if (cursorTokenIndex !== undefined) {
        // Subtracting 2, because of whitespace token
        const previousToken = tokenStream.get(cursorTokenIndex - 2);
        const { tokens, rules } = core.collectCandidates(cursorTokenIndex);
        const _a = generateSuggestionsFromRules(rules, cursorTokenIndex, previousToken), { suggestColumns } = _a, suggestionsFromRules = __rest(_a, ["suggestColumns"]);
        result = Object.assign(Object.assign({}, result), suggestionsFromRules);
        tokens.forEach((_, tokenType) => {
            var _a;
            // Literal keyword names are quoted
            const name = (_a = parser.vocabulary.getLiteralName(tokenType)) === null || _a === void 0 ? void 0 : _a.replace(quotesRegex, '$1');
            if (name) {
                suggestKeywords.push({
                    value: name,
                });
            }
        });
        const cursorIndex = getCursorIndex(query, cursor);
        const currentStatement = getCurrentStatement(query, cursorIndex);
        if (suggestColumns) {
            const modifiedQuery = modifyInvalidQuery(currentStatement.statement, currentStatement.cursorIndex);
            const inputStream = CharStreams.fromString(modifiedQuery);
            const lexer = new MySqlLexer(inputStream);
            const tokenStream = new CommonTokenStream(lexer);
            const parser = new MySqlParser(tokenStream);
            parser.removeErrorListeners();
            const parseTree = parser.root();
            const visitor = new SymbolTableVisitor();
            visitor.visit(parseTree);
            const tables = visitor.symbolTable.getNestedSymbolsOfTypeSync(TableSymbol);
            if (tables.length) {
                result.suggestColumns = {
                    tables: tables.map((tableSymbol) => ({
                        name: tableSymbol.name,
                        alias: tableSymbol.alias,
                    })),
                };
            }
        }
        result.suggestTemplates = shouldSuggestTemplates(currentStatement.statement, currentStatement.cursorIndex);
    }
    result.suggestKeywords = suggestKeywords;
    return result;
}
